import itertools


class Fieldsets:

    def __init__(self, *args):
        self.fieldsets = []
        for fieldset in args:
            self.fieldsets.append(fieldset)

    def get_fieldsets(self, request, obj):
        return self.fieldsets

    def get_fields_names(self, request, obj):
        result = []
        for fieldset in self.get_fieldsets(request, obj):
            result = itertools.chain(result, fieldset.get_fields_names(request, obj))
        return result

    def get_readonly_fields(self, request, obj):
        result = []
        for fieldset in self.get_fieldsets(request, obj):
            result = itertools.chain(result, fieldset.get_readonly_fields(request, obj))
        return result


class AbstractFieldset:

    def __init__(self, title):
        self.title = title

    def show(self, request, obj):
        return False

    def get_title(self, request, obj):
        return self.title

    def get_fields_names(self, request, obj):
        return ()

    def get_readonly_fields(self, request, obj):
        return ()


class Fieldset(AbstractFieldset):

    def __init__(self, title, fields):
        super(Fieldset, self).__init__(title)
        self.fields = fields

    def get_fields(self, request, obj):
        result = []
        for field in self.fields:
            if field.show(request, obj):
                result.append(field)
        return result

    def get_fields_names(self, request, obj):
        return [field.name for field in self.get_fields(request, obj)]

    def get_readonly_fields(self, request, obj):
        result = []
        for field in self.fields:
            if field.show(request, obj) and field.is_readonly(request, obj):
                result.append(field.name)
        return result

    def show(self, request, obj):
        return bool(self.get_fields(request, obj))


class InlineFieldset(AbstractFieldset):

    def __init__(self, title, inline_view):
        super(Fieldset, self).__init__(title)
        self.inline_view = inline_view

    def get_inline_view(self, request, obj):
        return self.inline_view

    def show(self, request, obj):
        return True


class Field:

    def __init__(self, name, readonly=None):
        self.name = name
        self.readonly = readonly

    def is_readonly(self, request, obj):
        return self.readonly

    def show(self, request, obj):
        return True



from django.contrib.auth.models import User

from germanium.test_cases.default import GermaniumTestCase
from germanium.tools import assert_true, assert_false, assert_equal, assert_not_equal
from germanium.tools.http import assert_http_redirect, assert_http_ok, assert_http_forbidden, assert_http_not_found

from is_core.auth.permissions import BasePermission, PermissionsSet

from .test_case import HelperTestCase, AsSuperuserTestCase


__all__ =(
    'PermissionsTestCase',
)


class ObjIsNonePermission(BasePermission):

    def has_permission(self, name, request, view, obj=None):
        return obj is None


class ObjIsNotNonePermission(BasePermission):

    def has_permission(self, name, request, view, obj=None):
        return obj is not None


class ObjIsStringPermission(BasePermission):

    def has_permission(self, name, request, view, obj=None):
        return isinstance(obj, str)


class PermissionsTestCase(GermaniumTestCase):

    def test_permissions_should_be_joined_with_operators(self):
        obj_is_none = ObjIsNonePermission()
        obj_is_not_none = ObjIsNotNonePermission()
        obj_is_string = ObjIsStringPermission()

        assert_true((obj_is_none | obj_is_not_none).has_permission('test', None, None, None))
        assert_true((obj_is_none | obj_is_not_none).has_permission('test', None, None, ''))
        assert_false((obj_is_none & obj_is_not_none).has_permission('test', None, None, None))
        assert_false((obj_is_none & obj_is_not_none).has_permission('test', None, None, ''))
        assert_true(((obj_is_none | obj_is_string) & obj_is_not_none).has_permission('test', None, None, ''))
        assert_true(((obj_is_none & obj_is_string) | obj_is_not_none).has_permission('test', None, None, ''))
        assert_false((obj_is_not_none | (obj_is_none & obj_is_string)).has_permission('test', None, None, None))
        assert_true((~obj_is_none).has_permission('test', None, None, ''))
        assert_false((~obj_is_not_none).has_permission('test', None, None, ''))
        assert_true((obj_is_none & ~obj_is_not_none).has_permission('test', None, None, None))

    def test_permissions_set_should_return_permissions_according_to_name(self):
        permission = PermissionsSet(
            none=ObjIsNonePermission(),
            not_none=ObjIsNotNonePermission(),
            string=ObjIsStringPermission(),
        )

        assert_true(permission.has_permission('none', None, None, None))
        assert_false(permission.has_permission('none', None, None, ''))
        assert_false(permission.has_permission('invalid', None, None, None))
        assert_false(permission.has_permission('not_none', None, None, None))
        assert_true(permission.has_permission('string', None, None, ''))


import types

from django.db.models.fields.files import FieldFile
from django.urls.resolvers import get_resolver

from germanium.test_cases.rest import RESTTestCase
from germanium.decorators import login_all, data_provider
from germanium.tools.trivials import assert_equal
from germanium.tools.http import assert_http_not_found, assert_http_accepted
from germanium.tools.rest import assert_valid_JSON_response, assert_valid_JSON_created_response

from is_core.tests.data_generator_test_case import DataGeneratorTestCase
from is_core.forms.boundfield import ReadonlyBoundField
from is_core.utils.compatibility import reverse

from pyston.utils import model_resources_to_dict

try:
    from auth_token.contrib.is_core.tests.auth_test_cases import RESTAuthMixin
except ImportError:
    class RESTAuthMixin:
        pass


def add_urls_to_resource(resource):

    def get_resource_list_url(self):
        return reverse('%s:api-%s' % (self.core.site_name, self.core.get_menu_group_pattern_name()))

    def get_resource_url(self, pk):
        return reverse('%s:api-resource-%s' % (
            self.core.site_name, self.core.get_menu_group_pattern_name()), args=(pk,)
        )

    resource._resource_url = types.MethodType(get_resource_url, resource)
    resource._resource_list_url = types.MethodType(get_resource_list_url, resource)
    return resource


@login_all
class TestRESTsAvailability(RESTAuthMixin, DataGeneratorTestCase, RESTTestCase):

    iteration = 5
    ignore_warnings = False

    @classmethod
    def setUpClass(cls):
        super(TestRESTsAvailability, cls).setUpClass()
        cls.rest_resources = cls.set_up_rest_resources()

    @classmethod
    def set_up_rest_resources(cls):
        # Must be here, because hanlers is not registered
        get_resolver().url_patterns
        resources_dict = model_resources_to_dict()
        rest_resources = []
        for resource_name, resource in resources_dict.items():
            if cls.get_model_label(resource.model) in cls.factories:
                add_urls_to_resource(resource)
                rest_resources.append((resource_name, resource, resource.model))
            elif not cls.ignore_warnings:
                cls.logger.warning('Model {} has not created factory class'.format(resource.model))

        return rest_resources

    def get_rest_resources(self):
        return self.rest_resources

    def get_serialized_data(self, request, resource, update=False):
        inst = self.new_instance(resource.model)

        form_class = resource(request)._generate_form_class(inst=update and inst or None)
        form = form_class(initial={'_user': self.logged_user.user, '_request': None}, instance=inst)
        data = {}

        for field in form:
            if not isinstance(field, ReadonlyBoundField):
                value = field.value()
                if isinstance(value, FieldFile):
                    value = None
                data[field.name] = value

        # Removed instance (must be created because FK)
        inst.delete()

        return data, inst

    @data_provider(get_rest_resources)
    def test_should_return_data_from_resource_list(self, resource_name, resource, model):
        list_url = resource._resource_list_url()

        if not resource(self.get_request_with_user(self.r_factory.get(list_url))).has_get_permission():
            return

        resp = self.get(list_url)
        started_total_count = int(resp['X-Total'])

        for i in range(self.iteration):
            self.new_instance(model)
            resp = self.get(list_url)
            assert_valid_JSON_response(resp, 'REST get list of model: {}\n response: {}'.format(model, resp))
            assert_equal(int(resp['X-Total']) - i, started_total_count + 1)

    @data_provider(get_rest_resources)
    def test_should_return_data_from_resource(self, resource_name, resource, model):
        for _ in range(self.iteration):
            inst = self.new_instance(model)

            url = resource._resource_url(inst.pk)

            if not resource(self.get_request_with_user(self.r_factory.get(url))).has_get_permission(obj=inst):
                break
            resp = self.get(url)
            assert_valid_JSON_response(resp, 'REST get of model: %s\n response: %s' % (model, resp))

    @data_provider(get_rest_resources)
    def test_should_delete_data_from_resource(self, resource_name, resource, model):
        for i in range(self.iteration):
            inst = self.new_instance(model)

            url = resource._resource_url(inst.pk)

            if not resource(self.get_request_with_user(self.r_factory.delete(url))).has_delete_permission(obj=inst):
                break

            resp = self.delete(url)
            assert_http_accepted(resp, 'REST delete of model: {}\n response: {}'.format(model, resp))
            resp = self.get(url)
            assert_http_not_found(resp, 'REST get (should not found) of model: {}\n response: {}'.format(model, resp))

    @data_provider(get_rest_resources)
    def test_should_create_data_of_resource(self, resource_name, resource, model):
        for _ in range(self.iteration):
            list_url = resource._resource_list_url()

            request = self.get_request_with_user(self.r_factory.post(list_url))
            if not resource(request).has_post_permission():
                break

            data, inst = self.get_serialized_data(request, resource)

            count_before = model._default_manager.all().count()

            resp = self.post(list_url, data=data)

            count_after = model._default_manager.all().count()
            assert_valid_JSON_created_response(resp, 'REST create of model: {}\n response: {}'.format(model, resp))
            assert_equal(count_before + 1, count_after)

    @data_provider(get_rest_resources)
    def test_should_update_data_of_resource(self, resource_name, resource, model):
        for _ in range(self.iteration):
            inst_from = self.new_instance(model)

            url = resource._resource_url(inst_from.pk)

            request = self.get_request_with_user(self.r_factory.put(url))

            if not resource(request).has_put_permission(obj=inst_from):
                break

            data, _ = self.get_serialized_data(request, resource, True)
            resp = self.put(url, data=data)
            assert_valid_JSON_response(resp, 'REST update of model: {}\n response: {}'.format(model, resp))



from django.views.generic.base import ContextMixin
from django.template.loader import render_to_string


class InlineView(ContextMixin):
    template_name = None

    def __init__(self, request, parent_view, parent_instance):
        self.request = request
        self.parent_view = parent_view
        self.parent_instance = parent_instance

    def render(self, context, title):
        template = self.template_name
        context.update(self.get_context_data(title=title))
        return render_to_string(template, context.flatten())

    def __copy__(self):
        return self

    def __deepcopy__(self, memo):
        return self



from django.http.response import HttpResponseRedirect, HttpResponse, Http404
from django.core.exceptions import ValidationError
from django.utils.encoding import force_text
from django.utils.translation import ugettext
from django.conf import settings

from is_core.exceptions import ResponseException
from is_core.exceptions.response import response_exception_factory
from is_core.utils.compatibility import resolve, MiddlewareMixin


class RequestKwargsMiddleware(MiddlewareMixin):

    def process_request(self, request):
        request.kwargs = resolve(request.path).kwargs


# Not working with pyston exceptions
class HTTPExceptionsMiddleware(MiddlewareMixin):

    def process_exception(self, request, exception):
        if isinstance(exception, ResponseException):
            return exception.get_response(request)
        if isinstance(exception, ValidationError):
            return response_exception_factory(request, 422, ugettext('Unprocessable Entity'), exception.messages)
        if not settings.DEBUG and isinstance(exception, Http404):
            return response_exception_factory(request, 404, ugettext('Not Found'), force_text(exception))



from django.template.base import Node, TemplateSyntaxError, NodeList, kwarg_re
from django.utils.encoding import force_text
from django.utils.functional import SimpleLazyObject

from is_core.utils import get_link_or_none
from is_core.utils.compatibility import Library


register = Library()


class Permissions:
    permissions_validators = {}

    def register_permission_validator(self, name, validator):
        self.permissions_validators[name] = validator

permissions = Permissions()


def get_obj(Model, pk):
    return Model.objects.get(pk=pk)


class PermissionNode(Node):

    def __init__(self, perm_name, vals, nodelist_true, nodelist_false):
        self.perm_name = perm_name
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.vals = vals

    def render(self, context):
        perm_name = self.perm_name.resolve(context, True)
        request = context.get('request')
        args = []
        for val in self.vals:
            args.append(val.resolve(context))

        core_permission = context.get('core_permission', {})
        view = context.get('view')
        if core_permission.has_permission(perm_name, request, view, *args):
            return self.nodelist_true.render(context)
        else:
            return self.nodelist_false.render(context)

    def validator_kwargs(self, request, validator):
        if 'pk' in request.kwargs:
            Model = getattr(validator.im_self, 'model')
            if Model:
                return {'obj': SimpleLazyObject(lambda: get_obj(Model, request.kwargs['pk']))}
        return {}


@register.tag
def has_permission(parser, token):
    bits = list(token.split_contents())
    if len(bits) < 2:
        raise TemplateSyntaxError("%r takes minimal one argument" % bits[0])
    end_tag = 'end' + bits[0]

    vals = []
    for bit in bits[2:]:
        vals.append(parser.compile_filter(bit))

    nodelist_true = parser.parse(('else', end_tag))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse((end_tag,))
        parser.delete_first_token()
    else:
        nodelist_false = NodeList()
    return PermissionNode(parser.compile_filter(bits[1]), vals, nodelist_true, nodelist_false)


class PermissionURLNode(Node):

    def __init__(self, pattern_name, kwargs, nodelist_true, nodelist_false):
        self.pattern_name = pattern_name
        self.nodelist_true = nodelist_true
        self.nodelist_false = nodelist_false
        self.kwargs = kwargs

    def render(self, context):
        pattern_name = self.pattern_name.resolve(context, True)
        request = context.get('request')
        kwargs = {
            force_text(k, 'ascii'): v.resolve(context)
            for k, v in self.kwargs.items()
        }

        link = get_link_or_none(pattern_name, request, view_kwargs=kwargs)
        if link:
            return self.nodelist_true.render(context.new({'url': link}))
        else:
            return self.nodelist_false.render(context)


@register.tag
def has_permission_to_url(parser, token):
    bits = list(token.split_contents())
    if len(bits) < 2:
        raise TemplateSyntaxError('"has_permission_to_url" takes minimal one argument')

    end_tag = 'end' + bits[0]

    kwargs = {}
    for bit in bits[2:]:
        match = kwarg_re.match(bit)
        if not match:
            raise TemplateSyntaxError('Malformed arguments to has_permission_to_url tag')
        name, value = match.groups()
        if name:
            kwargs[name] = parser.compile_filter(value)
        else:
            raise TemplateSyntaxError('Malformed arguments to has_permission_to_url tag')

    nodelist_true = parser.parse(('else', end_tag))
    token = parser.next_token()
    if token.contents == 'else':
        nodelist_false = parser.parse((end_tag,))
        parser.delete_first_token()
    else:
        nodelist_false = NodeList()
    return PermissionURLNode(parser.compile_filter(bits[1]), kwargs, nodelist_true, nodelist_false)



from django.forms.models import _get_foreign_key

from is_core.generic_views.table_views import TableViewMixin
from is_core.generic_views.inlines import InlineView


class InlineTableView(TableViewMixin, InlineView):

    template_name = 'is_core/forms/inline_table.html'
    fk_name = None

    def _get_api_url(self):
        from is_core.site import get_model_core
        return get_model_core(self.model).get_api_url(self.request)

    def _get_menu_group_pattern_name(self):
        from is_core.site import get_model_core
        return get_model_core(self.model).get_menu_group_pattern_name()

    def _get_list_filter(self):
        list_filter = super(InlineTableView, self)._get_list_filter()
        fk_name = _get_foreign_key(self.parent_instance.__class__, self.model, fk_name=self.fk_name).name
        list_filter['filter'] = filter = list_filter.get('filter', {})
        if 'filter' in list_filter:
            filter[fk_name] = self.parent_instance.pk
        return list_filter



class BasePermission:
    """
    Base IS core permission object which has only one method has_permission which must be implemented in descendant.
    """

    def has_permission(self, name, request, view, obj=None):
        """
        Checks if request has permission to the given action.

        Args:
            name (str): name of the permission
            request (django.http.request.HttpRequest): Django request object
            view (object): Django view or REST view object
            obj (object): Object that is related with the given request

        Returns:
            True/False
        """
        raise NotImplementedError

    def __and__(self, other):
        assert isinstance(other, BasePermission), 'Only permission instances can be joined'

        return AndPermission(self, other)

    def __or__(self, other):
        assert isinstance(other, BasePermission), 'Only permission instances can be joined'

        return OrPermission(self, other)

    def __invert__(self):
        return NotPermission(self)


class OperatorPermission(BasePermission):

    operator = None
    operator_function = None

    def __init__(self, *permissions):
        self._permissions = list(permissions)

    def has_permission(self, name, request, view, obj=None):
        return self.operator_function(
            permission.has_permission(name, request, view, obj=obj) for permission in self._permissions
        )

    def add(self, permission):
        assert isinstance(permission, BasePermission), 'Only permission instance can be added to the operator'

        self._permissions.append(permission)

    def __repr__(self):
        return '({})'.format(' {} '.format(self.operator).join(str(permission) for permission in self._permissions))

    def __iter__(self):
        for permission in self._permissions:
            yield permission


class AndPermission(OperatorPermission):
    """
    Helper for joining permissions with AND operator.
    """

    operator = '&'
    operator_function = all

    def __and__(self, other):
        assert isinstance(other, BasePermission), 'Only permission instances can be joined'

        self.add(other)
        return self


class OrPermission(OperatorPermission):
    """
    Helper for joining permissions with OR operator.
    """

    operator = '|'
    operator_function = any

    def __or__(self, other):
        assert isinstance(other, BasePermission), 'Only permission instances can be joined'

        self.add(other)
        return self


class NotPermission(BasePermission):

    def __init__(self, permission):
        self._permission = permission

    def has_permission(self, name, request, view, obj=None):
        return not self._permission.has_permission(name, request, view, obj=obj)


class PermissionsSet(BasePermission):
    """
    ``PermissionSet`` contains a set of permissions identified by name. Permission is granted if permission with the
    given name grants the access. Finally if no permission with the given name is found ``False`` is returned.
    """

    def __init__(self, **permissions_set):
        """
        Args:
            **permissions_set (BasePermission): permissions data
        """
        super().__init__()
        self._permissions = permissions_set

    def set(self, name, permission):
        """
        Adds permission with the given name to the set. Permission with the same name will be overridden.
        Args:
            name: name of the permission
            permission: permission instance
        """
        assert isinstance(permission, BasePermission), 'Only permission instances can be added to the set'

        self._permissions[name] = permission

    def has_permission(self, name, request, view, obj=None):
        permission = self._permissions.get(name, None)
        return permission is not None and permission.has_permission(name, request, view, obj=obj)

    def __iter__(self):
        for permission in self._permissions.values():
            yield permission


class IsAuthenticated(BasePermission):
    """
    Grant permission if user is authenticated and is active
    """

    def has_permission(self, name, request, view, obj=None):
        return request.user.is_authenticated and request.user.is_active


class IsSuperuser(BasePermission):
    """
    Grant permission if user is superuser
    """

    def has_permission(self, name, request, view, obj=None):
        return request.user.is_superuser


class IsAdminUser(BasePermission):
    """
    Grant permission if user is staff
    """

    def has_permission(self, name, request, view, obj=None):
        return request.user.is_staff


class AllowAny(BasePermission):
    """
    Grant permission every time
    """

    def has_permission(self, name, request, view, obj=None):
        return True


class CoreAllowed(BasePermission):
    """
    Grant permission if core permission with the name grants access
    """

    name = None

    def __init__(self, name=None):
        super().__init__()
        if name:
            self.name = name

    def has_permission(self, name, request, view, obj=None):
        return view.core.permission.has_permission(self.name or name, request, view, obj)


class CoreCreateAllowed(CoreAllowed):
    """
    Grant permission if core create permission grants access
    """

    name = 'create'


class CoreReadAllowed(CoreAllowed):
    """
    Grant permission if core read permission grants access
    """

    name = 'read'


class CoreUpdateAllowed(CoreAllowed):
    """
    Grant permission if core update permission grants access
    """

    name = 'update'


class CoreDeleteAllowed(CoreAllowed):
    """
    Grant permission if core delete permission grants access
    """

    name = 'delete'



from django.utils.encoding import force_text
from django.utils.safestring import mark_safe

from is_core.utils.compatibility import BoundField

from .widgets import SmartWidgetMixin
from .utils import ReadonlyValue


class SmartBoundField(BoundField):

    def as_widget(self, widget=None, attrs=None, only_initial=False):
        """
        Renders the field by rendering the passed widget, adding any HTML
        attributes passed as attrs.  If no widget is specified, then the
        field's default widget will be used.
        """
        if not widget:
            widget = self.field.widget

        if self.field.localize:
            widget.is_localized = True

        attrs = attrs or {}
        attrs = self.build_widget_attrs(attrs, widget)
        auto_id = self.auto_id
        if auto_id and 'id' not in attrs and 'id' not in widget.attrs:
            if not only_initial:
                attrs['id'] = auto_id
            else:
                attrs['id'] = self.html_initial_id

        if not only_initial:
            name = self.html_name
        else:
            name = self.html_initial_name
        if isinstance(widget, SmartWidgetMixin) and hasattr(self.form, '_request'):
            return force_text(widget.smart_render(self.form._request, name, self.value(), self.initial,
                                                  self.form, attrs=attrs))
        else:
            return force_text(widget.render(name, self.value(), attrs=attrs))


class ReadonlyBoundField(SmartBoundField):

    type = 'readonly'
    is_readonly = True

    def __init__(self, form, field, name):
        from .fields import SmartReadonlyField

        if isinstance(field, SmartReadonlyField):
            field._set_readonly_field(name, form)
        super(ReadonlyBoundField, self).__init__(form, field, name)

    def as_widget(self, widget=None, attrs=None, only_initial=False):
        if not widget:
            widget = self.field.widget

        if widget.is_hidden:
            return mark_safe('')
        else:
            return super(ReadonlyBoundField, self).as_widget(
                self.form._get_readonly_widget(self.name, self.field, widget), attrs, only_initial
            )

    def as_hidden(self, attrs=None, **kwargs):
        """
        Returns a string of HTML for representing this as an <input type="hidden">.
        Because readonly has not hidden input there must be returned empty string.
        """
        return mark_safe('')

    @property
    def initial(self):
        data = self.form.initial.get(self.name, self.field.initial)
        if callable(data):
            data = data()

        value = self.field.prepare_value(data)

        if hasattr(self.form, 'humanized_data') and self.name in self.form.humanized_data:
            humanized_value = self.form.humanized_data.get(self.name)
            return ReadonlyValue(value, humanized_value)
        return value

    def value(self):
        return self.initial



from pyston.forms import RESTModelForm

from django import forms
from django.utils.translation import ugettext_lazy as _


class UserForm(RESTModelForm):
    password = forms.CharField(label=_('Password'), widget=forms.PasswordInput)

    def __init__(self, instance=None, *args, **kwargs):
        super(UserForm, self).__init__(instance=instance, *args, **kwargs)
        if instance:
            self.fields['password'].required = False

    def save(self, commit=True):
        user = super(UserForm, self).save(commit=False)
        if self.cleaned_data.get('password'):
            user.set_password(self.cleaned_data['password'])
        if commit:
            user.save()
        return user

    class Meta:
        exclude = ('last_login', 'password', 'is_staff', 'is_active', 'date_joined', 'groups', 'user_permissions')



from django import forms

from pyston.filters.default_filters import (
    DateFilter, ManyToManyFieldFilter, OPERATORS, ForeignKeyFilter, ForeignObjectRelFilter
)

from is_core.forms.widgets import DateRangeFilterWidget, DateTimeRangeFilterWidget, RestrictedSelectWidget

from . import UIFilterMixin, FilterChoiceIterator


class DateRangeFilter(UIFilterMixin, DateFilter):
    """
    UI filter for date field that provides possibility to filter date from/to
    """

    widget = DateRangeFilterWidget


class DateTimeRangeFilter(UIFilterMixin, DateFilter):
    """
    UI filter for datetime field that provides possibility to filter date from/to
    """

    widget = DateTimeRangeFilterWidget


class RelatedUIFilter(UIFilterMixin):
    """
    Helper for all filters of related field.
    """

    def _update_widget_choices(self, widget):
        """
        Updates widget choices with special choice iterator that removes blank values and adds none value to clear
        filter data.
        :param widget: widget with choices
        :return: updated widget with filter choices
        """

        widget.choices = FilterChoiceIterator(widget.choices, self.field)
        return widget

    def get_operator(self, widget):
        """
        Because related form field can be restricted (if choices is too much it is used textarea without select box)
        for situation without choices (textarea) is used contains operator.
        :param widget: restricted widget
        :return: operator that will be used for filtering
        """
        return OPERATORS.CONTAINS if widget.is_restricted else OPERATORS.EQ


class UIForeignKeyFilter(RelatedUIFilter, ForeignKeyFilter):

    def get_widget(self, request):
        """
        Field widget is replaced with "RestrictedSelectWidget" because we not want to use modified widgets for
        filtering.
        """
        return self._update_widget_choices(self.field.formfield(widget=RestrictedSelectWidget).widget)


class UIManyToManyFieldFilter(RelatedUIFilter, ManyToManyFieldFilter):

    def get_widget(self, request):
        """
        Field widget is replaced with "RestrictedSelectWidget" because "MultipleChoiceField" is not optional for
        filtering purposes.
        """
        return self._update_widget_choices(self.field.formfield(widget=RestrictedSelectWidget).widget)


class UIForeignObjectRelFilter(RelatedUIFilter, ForeignObjectRelFilter):

    def get_widget(self, request):
        """
        Table view is not able to get form field from reverse relation.
        Therefore this widget returns similar form field as direct relation (ModelChoiceField).
        Because there is used "RestrictedSelectWidget" it is returned textarea or selectox with choices according to
        count objects in the queryset.
        """
        return self._update_widget_choices(
            forms.ModelChoiceField(
                widget=RestrictedSelectWidget, queryset=self.field.related_model._default_manager.all()
            ).widget
        )



import json

from django.utils.encoding import force_text

from germanium.tools import assert_true, assert_not_equal
from germanium.test_cases.client import ClientTestCase
from germanium.decorators import login
from germanium.crawler import Crawler, LinkExtractor, HtmlLinkExtractor as OriginalHtmlLinkExtractor


def flatt_list(iterable_value):
    flatten_list = []

    for val in iterable_value:
        if isinstance(val, list):
            flatten_list += val
        else:
            flatten_list.append(val)
    return flatten_list


class JSONLinkExtractor(LinkExtractor):

    def _extract_web_links(self, data):
        return flatt_list(data.values())

    def _extract_rest_links(self, data):
        links = []
        for rest_link in data.values():
            if 'GET' in rest_link['methods']:
                links += flatt_list([rest_link['url']])
        return links

    def _extract_from_dict(self, data):
        links = []
        for key, val in data.items():
            if key == '_web_links':
                links += self._extract_web_links(val)
            elif key == '_rest_links':
                links += self._extract_rest_links(val)
            elif isinstance(val, (list, tuple)):
                links += self._extract_from_list(val)
            elif isinstance(val, dict):
                links += self._extract_from_dict(val)
        return links

    def _extract_from_list(self, data):
        links = []
        for val in data:
            if isinstance(val, dict):
                links += self._extract_from_dict(val)
            elif isinstance(val, (list, tuple)):
                links += self._extract_from_list(val)
        return links

    def extract(self, content):
        data = json.loads(content)
        if isinstance(data, dict):
            links = self._extract_from_dict(data)
        elif isinstance(data, (list, tuple)):
            links = self._extract_from_list(data)
        return links


class HTMLLinkExtractor(OriginalHtmlLinkExtractor):
    link_attr_names = ('href', 'src', 'data-resource')


class TextPlainSnippetsExtractor(LinkExtractor):

    def extract(self, content):
        links = []

        try:
            data = json.loads(content)

            html_extractor = HTMLLinkExtractor()
            for html in data.get('snippets', {}).values():
                links += html_extractor.extract(html)
        except ValueError:
            # I text/plain is not snippet return empty links
            pass

        return links


class CrawlerTestCase(ClientTestCase):

    REST_BASE = None
    exclude_urls = ()

    def get_users(self):
        raise NotImplementedError

    def get_exlude_urls(self):
        return list(self.exclude_urls) + ['/logout/']

    @login(users_generator='get_users')
    def test_crawler(self):
        self.logger.info('\n---------------------------')
        self.logger.info('Test crawling with logged user %s' % self.logged_user.user)

        tested_urls = []
        failed_urls = []

        def pre_request(url, referer, headers):
            if url.startswith('/api/'):
                headers['HTTP_X_FIELDS'] = '_rest_links,_web_links'
            if self.REST_BASE:
                headers['HTTP_X_BASE'] = str(self.REST_BASE)
            return url, headers

        def post_response(url, referer, resp, exception):
            tested_urls.append(url)
            assert_true(exception is None, msg='Received exception %s, url %s' % (force_text(exception), url))
            if resp.status_code != 200:
                failed_urls.append(url)
                self.logger.warning('Response code for url %s from referer %s should be 200 but code is %s, user %s' %
                                    (url, referer, resp.status_code, self.logged_user.user))
            assert_not_equal(resp.status_code, 500, msg='Response code for url %s from referer %s is 500, user %s' %
                             (url, referer, self.logged_user.user))
        Crawler(self.c, ('/',), self.get_exlude_urls(), pre_request, post_response,
                extra_link_extractors={'application/json; charset=utf-8': JSONLinkExtractor(),
                                       'text/plain': TextPlainSnippetsExtractor(),
                                       'default': HTMLLinkExtractor()}).run()

        self.logger.info('Completed with tested %s urls (warnings %s)' % (len(tested_urls), len(failed_urls)))
        self.logger.info('---------------------------')
